/*
 * Copyright 2017-2020, Schlumberger
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.opengroup.osdu.register.util;

import com.sun.jersey.api.client.ClientResponse;
import org.junit.Test;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.*;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

public abstract class DeleteApiTestTemplate extends BaseTestTemplate {

    public DeleteApiTestTemplate(RestDescriptor deleteDescriptor, RestDescriptor createDescriptor) {
        super(deleteDescriptor);
        this.createDescriptor = createDescriptor;
    }

    private RestDescriptor createDescriptor;
    private String id = "deletetest" + System.currentTimeMillis();

    @Override
    protected String getId() {
        return id;
    }

    @Override
    protected void validate20XResponse(ClientResponse response, RestDescriptor descriptor) {
        //do nothing
    }

    @Override
    protected void deleteResource() {
        //do nothingthis api deletes the resources
    }

    @Override
    protected void createResource() throws Exception {
        createDescriptor.run(getId(), testUtils.getOpsAccessToken());
    }

    @Override
    protected int expectedOkResponseCode() {
        return 204;
    }

    @Test
    public void should_beAbleToRunApiInParallel() throws Exception {
        createResource();
        ExecutorService executor = Executors.newFixedThreadPool(10);
        List<Callable<ClientResponse>> tasks = new ArrayList<>();

        for (int i = 0; i < 10; i++) {
            Callable<ClientResponse> task = () -> {
                try {
                    return descriptor.run(getId(), testUtils.getOpsAccessToken());
                } catch (Exception ex) {
                    return null;
                }
            };
            tasks.add(task);
        }

        List<Future<ClientResponse>> responses = executor.invokeAll(tasks);
        executor.shutdown();
        executor.awaitTermination(30, TimeUnit.SECONDS);
        List<Integer> errors = new ArrayList<>();
        int sucessResponseCount = 0;
        int non404or409ErrorResponseCount = 0;
        for (Future<ClientResponse> future : responses) {
            if (future.get() == null)
                fail(String.format("Failed to get response in time for %s %s    %s", descriptor.getHttpMethod(), descriptor.getPath(), descriptor.getArg()));
            if (future.get().getStatus() == expectedOkResponseCode())
                sucessResponseCount++;
            else if (future.get().getStatus() != 409 && future.get().getStatus() != 404) {
                non404or409ErrorResponseCount++;
                errors.add(future.get().getStatus());
            }
        }

        assertTrue(error("Expected 1 successful response. Actual " + sucessResponseCount), sucessResponseCount <= 1);
        assertEquals(error("Unexpected error response returned " + errors.toString()), 0, non404or409ErrorResponseCount);
    }
}
